@@ -1,9 +1,10 @@
 import sys
 
-# True if we are running on Python 3.
-PY3 = sys.version_info[0] == 3
-PY32 = PY3 and sys.version_info[1] == 2
-PY2 = not PY3
+PY_MAJOR = sys.version_info[0]
+PY_MINOR = sys.version_info[1]
+PY2 = PY_MAJOR == 2
+PY3 = PY_MAJOR == 3
+PY32 = PY3 and PY_MINOR == 2
 
 try:
     bytes = bytes
@@ -30,3 +31,11 @@ try:
     unichr = unichr
 except NameError:
     unichr = chr
+
+
+try:
+    # Python3
+    import queue
+except ImportError:
+    # Python2
+    import Queue as queue
