@@ -20,12 +20,13 @@ objects that implement the reduce protocol::
 
 """
 
-import copy
-import sys
-import datetime
-import time
 import collections
+import copy
+import datetime
 import decimal
+import re
+import sys
+import time
 
 from jsonpickle import util
 from jsonpickle.compat import unicode
@@ -44,10 +45,12 @@ class Registry(object):
         :param handler: The custom handler class
 
         """
-        self._handlers[cls] = handler
+        key = util.importable_name(cls)
+        self._handlers[key] = handler
+
+    def get(self, class_name):
+        return self._handlers.get(class_name)
 
-    def get(self, cls):
-        return self._handlers.get(cls)
 
 registry = Registry()
 register = registry.register
@@ -111,8 +114,8 @@ class DatetimeHandler(BaseHandler):
         data['__reduce__'] = (flatten(cls, reset=False), args)
         return data
 
-    def restore(self, obj):
-        cls, args = obj['__reduce__']
+    def restore(self, data):
+        cls, args = data['__reduce__']
         unpickler = self.context
         restore = unpickler.restore
         cls = restore(cls, reset=False)
@@ -126,6 +129,19 @@ DatetimeHandler.handles(datetime.date)
 DatetimeHandler.handles(datetime.time)
 
 
+class RegexHandler(BaseHandler):
+    """Flatten _sre.SRE_Pattern (compiled regex) objects"""
+
+    def flatten(self, obj, data):
+        data['pattern'] = obj.pattern
+        return data
+
+    def restore(self, data):
+        return re.compile(data['pattern'])
+
+RegexHandler.handles(type(re.compile('')))
+
+
 class SimpleReduceHandler(BaseHandler):
     """Follow the __reduce__ protocol to pickle an object.
 
@@ -138,9 +154,9 @@ class SimpleReduceHandler(BaseHandler):
         data['__reduce__'] = [flatten(i, reset=False) for i in obj.__reduce__()]
         return data
 
-    def restore(self, obj):
+    def restore(self, data):
         restore = self.context.restore
-        factory, args = [restore(i, reset=False) for i in obj['__reduce__']]
+        factory, args = [restore(i, reset=False) for i in data['__reduce__']]
         return factory(*args)
 
 
@@ -195,7 +211,7 @@ class QueueHandler(BaseHandler):
     def flatten(self, obj, data):
         return data
 
-    def restore(self, obj):
+    def restore(self, data):
         return queue.Queue()
 
 QueueHandler.handles(queue.Queue)
