@@ -7,7 +7,9 @@
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.
 
-from itertools import chain
+import warnings
+import sys
+from itertools import chain, islice
 
 import jsonpickle.util as util
 import jsonpickle.tags as tags
@@ -18,16 +20,24 @@ from jsonpickle.compat import unicode
 
 
 def encode(value,
-           unpicklable=False, make_refs=True, keys=False,
-           max_depth=None, reset=True,
-           backend=None, context=None):
+           unpicklable=False,
+           make_refs=True,
+           keys=False,
+           max_depth=None,
+           reset=True,
+           backend=None,
+           warn=False,
+           context=None,
+           max_iter=None):
     backend = _make_backend(backend)
     if context is None:
         context = Pickler(unpicklable=unpicklable,
                           make_refs=make_refs,
                           keys=keys,
                           backend=backend,
-                          max_depth=max_depth)
+                          max_depth=max_depth,
+                          warn=warn,
+                          max_iter=max_iter)
     return backend.encode(context.flatten(value, reset=reset))
 
 
@@ -41,12 +51,18 @@ def _make_backend(backend):
 class Pickler(object):
 
     def __init__(self,
-                unpicklable=True, make_refs=True, max_depth=None,
-                backend=None, keys=False):
+                 unpicklable=True,
+                 make_refs=True,
+                 max_depth=None,
+                 backend=None,
+                 keys=False,
+                 warn=False,
+                 max_iter=None):
         self.unpicklable = unpicklable
         self.make_refs = make_refs
         self.backend = _make_backend(backend)
         self.keys = keys
+        self.warn = warn
         ## The current recursion depth
         self._depth = -1
         ## The maximal recursion depth
@@ -55,6 +71,8 @@ class Pickler(object):
         self._objs = {}
         ## Avoids garbage collection
         self._seen = []
+        # maximum amount of items to take from a pickled iterator
+        self._max_iter = max_iter
 
     def reset(self):
         self._objs = {}
@@ -136,6 +154,10 @@ class Pickler(object):
         else:
             flatten_func = self._get_flattener(obj)
 
+        if flatten_func is None:
+            self._pickle_warning(obj)
+            return None
+
         return flatten_func(obj)
 
     def _list_recurse(self, obj):
@@ -175,7 +197,11 @@ class Pickler(object):
         if util.is_object(obj):
             return self._ref_obj_instance
 
-        # else, what else? (methods, functions, old style classes...)
+        if util.is_module_function(obj):
+            return self._flatten_function
+
+        # instance methods, lambdas, old style classes...
+        self._pickle_warning(obj)
         return None
 
     def _ref_obj_instance(self, obj):
@@ -197,19 +223,84 @@ class Pickler(object):
         has_class = hasattr(obj, '__class__')
         has_dict = hasattr(obj, '__dict__')
         has_slots = not has_dict and hasattr(obj, '__slots__')
+        has_getnewargs = hasattr(obj, '__getnewargs__')
+        has_getinitargs = hasattr(obj, '__getinitargs__')
+        has_reduce, has_reduce_ex = util.has_reduce(obj)
 
         # Support objects with __getstate__(); this ensures that
         # both __setstate__() and __getstate__() are implemented
         has_getstate = hasattr(obj, '__getstate__')
 
-        if has_class and not util.is_module(obj):
-            module, name = _getclassdetail(obj)
+        if has_class:
+            cls = obj.__class__
+        else:
+            cls = type(obj)
+
+        # Check for a custom handler
+        class_name = util.importable_name(cls)
+        handler = handlers.get(class_name)
+        if handler is not None:
             if self.unpicklable:
-                data[tags.OBJECT] = '%s.%s' % (module, name)
-            # Check for a custom handler
-            handler = handlers.get(obj.__class__)
-            if handler is not None:
-                return handler(self).flatten(obj, data)
+                data[tags.OBJECT] = class_name
+            return handler(self).flatten(obj, data)
+
+        reduce_val = None
+        if has_class and not util.is_module(obj):
+            if self.unpicklable:
+                class_name = util.importable_name(cls)
+                data[tags.OBJECT] = class_name
+
+            # test for a reduce implementation, and redirect before doing anything else
+            # if that is what reduce requests
+            if has_reduce_ex:
+                try:
+                    # we're implementing protocol 2
+                    reduce_val = obj.__reduce_ex__(2)
+                except TypeError:
+                    # A lot of builtin types have a reduce which just raises a TypeError
+                    # we ignore those
+                    pass
+
+            if has_reduce and not reduce_val:
+                try:
+                    reduce_val = obj.__reduce__()
+                except TypeError:
+                    # A lot of builtin types have a reduce which just raises a TypeError
+                    # we ignore those
+                    pass
+
+            if reduce_val:
+                try:
+                    # At this stage, we only handle the case where __reduce__ returns a string
+                    # other reduce functionality is implemented further down
+                    if isinstance(reduce_val, (str, unicode)):
+                        varpath = iter(reduce_val.split('.'))
+                        # curmod will be transformed by the loop into the value to pickle
+                        curmod = sys.modules[next(varpath)]
+                        for modname in varpath:
+                            curmod = getattr(curmod, modname)
+                            # replace obj with value retrieved
+                        return self._flatten(curmod)
+                except KeyError:
+                    # well, we can't do anything with that, so we ignore it
+                    pass
+
+            if has_getnewargs:
+                data[tags.NEWARGS] = self._flatten(obj.__getnewargs__())
+
+            if has_getinitargs:
+                data[tags.INITARGS] = self._flatten(obj.__getinitargs__())
+
+        if has_getstate:
+            try:
+                state = obj.__getstate__()
+            except TypeError:
+                # Has getstate but it cannot be called, e.g. file descriptors
+                # in Python3
+                self._pickle_warning(obj)
+                return None
+            else:
+                return self._getstate(state, data)
 
         if util.is_module(obj):
             if self.unpicklable:
@@ -221,31 +312,66 @@ class Pickler(object):
 
         if util.is_dictionary_subclass(obj):
             self._flatten_dict_obj(obj, data)
-            if has_getstate:
-                self._getstate(obj, data)
             return data
 
-        if has_dict:
-            # Support objects that subclasses list and set
-            if util.is_sequence_subclass(obj):
-                return self._flatten_sequence_obj(obj, data)
-
-            if has_getstate:
-                return self._getstate(obj, data)
-
-            # hack for zope persistent objects; this unghostifies the object
-            getattr(obj, '_', None)
-            return self._flatten_dict_obj(obj.__dict__, data)
-
         if util.is_sequence_subclass(obj):
             return self._flatten_sequence_obj(obj, data)
 
         if util.is_noncomplex(obj):
             return [self._flatten(v) for v in obj]
 
+        if util.is_iterator(obj):
+            # force list in python 3
+            data[tags.ITERATOR] = list(map(self._flatten, islice(obj, self._max_iter)))
+            return data
+
+        if reduce_val and not isinstance(reduce_val, (str, unicode)):
+            # at this point, reduce_val should be some kind of iterable
+            # pad out to len 5
+            rv_as_list = list(reduce_val)
+            insufficiency = 5 - len(rv_as_list)
+            if insufficiency:
+                rv_as_list+=[None]*insufficiency
+
+            if rv_as_list[0].__name__ == '__newobj__':
+                rv_as_list[0] = tags.NEWOBJ
+
+            data[tags.REDUCE] = list(map(self._flatten, rv_as_list))
+
+            # lift out iterators, so we don't have to iterator and uniterator their content
+            # on unpickle
+            if data[tags.REDUCE][3]:
+                data[tags.REDUCE][3] = data[tags.REDUCE][3][tags.ITERATOR]
+
+            if data[tags.REDUCE][4]:
+                data[tags.REDUCE][4] = data[tags.REDUCE][4][tags.ITERATOR]
+
+            return data
+
+        if has_dict:
+            # Support objects that subclasses list and set
+            if util.is_sequence_subclass(obj):
+                return self._flatten_sequence_obj(obj, data)
+
+            # hack for zope persistent objects; this unghostifies the object
+            getattr(obj, '_', None)
+            return self._flatten_dict_obj(obj.__dict__, data)
+
         if has_slots:
             return self._flatten_newstyle_with_slots(obj, data)
 
+        self._pickle_warning(obj)
+        return None
+
+    def _flatten_function(self, obj):
+        if self.unpicklable:
+            data = {tags.FUNCTION: util.importable_name(obj)}
+        else:
+            data = None
+
+        return data
+
+
     def _flatten_dict_obj(self, obj, data=None):
         """Recursively call flatten() and return json-friendly dict
         """
@@ -270,19 +396,30 @@ class Pickler(object):
 
         return data
 
-    def _flatten_newstyle_with_slots(self, obj, data):
-        """Return a json-friendly dict for new-style objects with __slots__.
-        """
-        allslots = [getattr(cls, '__slots__', tuple())
-                        for cls in type(obj).mro()]
+    def _flatten_obj_attrs(self, obj, attrs, data):
         flatten = self._flatten_key_value_pair
-        for k in chain(*allslots):
+        ok = False
+        for k in attrs:
             try:
                 value = getattr(obj, k)
+                flatten(k, value, data)
             except AttributeError:
                 # The attribute may have been deleted
                 continue
-            flatten(k, value, data)
+            ok = True
+        return ok
+
+    def _flatten_newstyle_with_slots(self, obj, data):
+        """Return a json-friendly dict for new-style objects with __slots__.
+        """
+        allslots = [_wrap_string_slot(getattr(cls, '__slots__', tuple()))
+                        for cls in obj.__class__.mro()]
+
+        if not self._flatten_obj_attrs(obj, chain(*allslots), data):
+            attrs = [x for x in dir(obj)
+                        if not x.startswith('__') and not x.endswith('__')]
+            self._flatten_obj_attrs(obj, attrs, data)
+
         return data
 
     def _flatten_key_value_pair(self, k, v, data):
@@ -315,13 +452,18 @@ class Pickler(object):
         return data
 
     def _getstate(self, obj, data):
-        state = self._flatten_obj(obj.__getstate__())
+        state = self._flatten_obj(obj)
         if self.unpicklable:
             data[tags.STATE] = state
         else:
             data = state
         return data
 
+    def _pickle_warning(self, obj):
+        if self.warn:
+            msg = 'jsonpickle cannot pickle %r: replaced with None' % obj
+            warnings.warn(msg)
+
 
 def _mktyperef(obj):
     """Return a typeref dictionary
@@ -330,27 +472,12 @@ def _mktyperef(obj):
     {'py/type': '__builtin__.AssertionError'}
 
     """
-    return {tags.TYPE: '%s.%s' %
-            (util.translate_module_name(obj.__module__), obj.__name__)}
+    return {tags.TYPE: util.importable_name(obj)}
 
 
-def _getclassdetail(obj):
-    """Helper class to return the class of an object.
-
-    >>> class Example(object): pass
-    >>> _getclassdetail(Example())
-    ('jsonpickle.pickler', 'Example')
-    >>> _getclassdetail(25)
-    ('__builtin__', 'int')
-    >>> _getclassdetail(None)
-    ('__builtin__', 'NoneType')
-    >>> _getclassdetail(False)
-    ('__builtin__', 'bool')
-    >>> _getclassdetail(AttributeError)
-    ('__builtin__', 'type')
-
+def _wrap_string_slot(string):
+    """Converts __slots__ = 'a' into __slots__ = ('a',)
     """
-    cls = obj.__class__
-    module = getattr(cls, '__module__')
-    name = getattr(cls, '__name__')
-    return util.translate_module_name(module), name
+    if isinstance(string, (str, unicode)):
+        return (string,)
+    return string
