@@ -8,16 +8,39 @@ that need to be specially handled.
 """
 from jsonpickle.compat import set
 
+
+FUNCTION = 'py/function'
 ID = 'py/id'
-OBJECT = 'py/object'
-TYPE = 'py/type'
-REPR = 'py/repr'
-REF = 'py/ref'
-TUPLE = 'py/tuple'
-SET = 'py/set'
-SEQ = 'py/seq'
-STATE = 'py/state'
+INITARGS = 'py/initargs'
+ITERATOR = 'py/iterator'
 JSON_KEY = 'json://'
+NEWARGS = 'py/newargs'
+NEWOBJ = 'py/newobj'
+OBJECT = 'py/object'
+REDUCE = 'py/reduce'
+REF = 'py/ref'
+REPR = 'py/repr'
+SEQ = 'py/seq'
+SET = 'py/set'
+STATE = 'py/state'
+TUPLE = 'py/tuple'
+TYPE = 'py/type'
 
 # All reserved tag names
-RESERVED = set([OBJECT, TYPE, REPR, REF, TUPLE, SET, SEQ, STATE])
+RESERVED = set([
+    FUNCTION,
+    ID,
+    INITARGS,
+    ITERATOR,
+    NEWARGS,
+    NEWOBJ,
+    OBJECT,
+    REDUCE,
+    REF,
+    REPR,
+    SEQ,
+    SET,
+    STATE,
+    TUPLE,
+    TYPE,
+])
