@@ -10,6 +10,8 @@
 determining the type of an object.
 """
 import base64
+import collections
+import io
 import operator
 import time
 import types
@@ -20,6 +22,8 @@ from jsonpickle.compat import unicode
 from jsonpickle.compat import long
 from jsonpickle.compat import PY3
 
+if not PY3:
+    import __builtin__
 
 SEQUENCES = (list, set, tuple)
 SEQUENCES_SET = set(SEQUENCES)
@@ -147,7 +151,8 @@ def is_sequence_subclass(obj):
     >>> is_sequence_subclass(Temp())
     True
     """
-    return ((issubclass(obj.__class__, SEQUENCES) or
+    return (hasattr(obj, '__class__') and
+            (issubclass(obj.__class__, SEQUENCES) or
                 is_list_like(obj)) and
             not is_sequence(obj))
 
@@ -196,6 +201,25 @@ def is_function(obj):
                      'method-wrapper'))
 
 
+def is_module_function(obj):
+    """Return True if `obj` is a module-global function
+
+    >>> import os
+    >>> is_module_function(os.path.exists)
+    True
+
+    >>> is_module_function(lambda: None)
+    False
+
+    """
+
+    return (hasattr(obj, '__class__') and
+            obj.__class__ is types.FunctionType and
+            hasattr(obj, '__module__') and
+            hasattr(obj, '__name__') and
+            obj.__name__ != '<lambda>')
+
+
 def is_module(obj):
     """Returns True if passed a module
 
@@ -216,12 +240,15 @@ def is_picklable(name, value):
 
     >>> def foo(): pass
     >>> is_picklable('foo', foo)
+    True
+
+    >>> is_picklable('foo', lambda: None)
     False
 
     """
     if name in tags.RESERVED:
         return False
-    return not is_function(value)
+    return is_module_function(value) or not is_function(value)
 
 
 def is_installed(module):
@@ -236,7 +263,7 @@ def is_installed(module):
     try:
         __import__(module)
         return True
-    except ImportError as e:
+    except ImportError:
         return False
 
 
@@ -244,6 +271,78 @@ def is_list_like(obj):
     return hasattr(obj, '__getitem__') and hasattr(obj, 'append')
 
 
+def is_iterator(obj):
+    is_file = False
+    if not PY3:
+        is_file = isinstance(obj, __builtin__.file)
+
+    return (isinstance(obj, collections.Iterator) and
+            not isinstance(obj, io.IOBase) and not is_file)
+
+
+def is_reducible(obj):
+    """
+    Returns false if of a type which have special casing, and should not have their
+    __reduce__ methods used
+    """
+    return (not (is_list(obj) or is_list_like(obj) or is_primitive(obj) or
+                 is_dictionary(obj) or is_sequence(obj) or is_set(obj) or is_tuple(obj) or
+                 is_dictionary_subclass(obj) or is_sequence_subclass(obj) or is_noncomplex(obj)
+                 or is_function(obj) or is_module(obj) or type(obj) is object or obj is object
+                 or (is_type(obj) and obj.__module__ == 'datetime')))
+
+
+def in_dict(obj, key, default=False):
+    """
+    Returns true if key exists in obj.__dict__; false if not in.
+    If obj.__dict__ is absent, return default
+    """
+    return (key in obj.__dict__) if getattr(obj, '__dict__', None) else default
+
+
+def in_slots(obj, key, default=False):
+    """
+    Returns true if key exists in obj.__slots__; false if not in.
+    If obj.__slots__ is absent, return default
+    """
+    return (key in obj.__slots__) if getattr(obj, '__slots__', None) else default
+
+
+def has_reduce(obj):
+    """
+    Tests if __reduce__ or __reduce_ex__ exists in the object dict or
+    in the class dicts of every class in the MRO *except object*.
+
+    Returns a tuple of booleans (has_reduce, has_reduce_ex)
+    """
+
+    if not is_reducible(obj) or is_type(obj):
+        return (False, False)
+
+    has_reduce = False
+    has_reduce_ex = False
+
+    REDUCE = '__reduce__'
+    REDUCE_EX = '__reduce_ex__'
+
+    # For object instance
+    has_reduce = in_dict(obj, REDUCE)
+    has_reduce_ex = in_dict(obj, REDUCE_EX)
+
+    has_reduce = has_reduce or in_slots(obj, REDUCE)
+    has_reduce_ex = has_reduce_ex or in_slots(obj, REDUCE_EX)
+
+    # turn to the MRO
+    for base in type(obj).__mro__:
+        if is_reducible(base):
+            has_reduce = has_reduce or in_dict(base, REDUCE)
+            has_reduce_ex = has_reduce_ex or in_dict(base, REDUCE_EX)
+        if has_reduce_ex and has_reduce_ex:
+            return (True, True)
+
+    return (has_reduce, has_reduce_ex)
+
+
 def translate_module_name(module):
     """Rename builtin modules to a consistent (Python2) module name
 
@@ -279,6 +378,33 @@ def untranslate_module_name(module):
     return module
 
 
+def importable_name(cls):
+    """
+    >>> class Example(object):
+    ...     pass
+
+    >>> ex = Example()
+    >>> importable_name(ex.__class__)
+    'jsonpickle.util.Example'
+
+    >>> importable_name(type(25))
+    '__builtin__.int'
+
+    >>> importable_name(None.__class__)
+    '__builtin__.NoneType'
+
+    >>> importable_name(False.__class__)
+    '__builtin__.bool'
+
+    >>> importable_name(AttributeError)
+    '__builtin__.AttributeError'
+
+    """
+    name = cls.__name__
+    module = translate_module_name(cls.__module__)
+    return '%s.%s' % (module, name)
+
+
 def b64encode(data):
     payload = base64.b64encode(data)
     if PY3 and type(payload) is bytes:
